package PocketIO::Transport::XHRPolling;

use strict;
use warnings;

use base 'PocketIO::Transport::BasePolling';

sub name {'xhr-polling'}

1;
__END__

=head1 NAME

PocketIO::XHRPolling - XHRPolling transport

=head1 DESCRIPTION

L<PocketIO::XHRPolling> is a C<xhr-polling> transport
implementation.

=head1 METHODS

=head2 C<name>

=head2 C<dispatch>

=cut
